/* Copyright © 2013, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#define _GNU_SOURCE
#include <dlfcn.h>

#include <stdio.h>

/* The purpose of this program is very simple. It dynamically loads
 * libGL.so.1 and then prints out the filename of the file which was
 * loaded.
 *
 * We use this in Glaze in order to determine the libGL.so.1 library
 * that would be loaded if it weren't for us messing around with
 * LD_LIBRARY_PATH causing Glaze's libGL.so.1 to be loaded instead.
 *
 * Using a program like this allows us to directly inspect what the
 * dynamic loader actually does. This is much more reliable than
 * trying to recreate its search-path logic which would requrie at
 * least:
 *
 *	* Parsing LD_LIBRARY_PATH
 *
 *	* Correctly interpreting $LIB within LD_LIBRARY_PATH
 *
 *	* Knowing the canonical name for the current architecture for
 *        *$LIB
 *
 *	* Parsing /etc/ld.so.conf entries
 *
 *	* etc. etc.
 */
int
main (void)
{
	Dl_info info;
	void *handle;
	void *function;

	handle = dlopen ("libGL.so.1", RTLD_NOW);
	if (handle == NULL) {
		fprintf (stderr, "glaze-find-libgl: Failed to dlopen libGL.so.1\n");
		return 1;
	}

	function = dlsym (handle, "glClear");
	if (function == NULL) {
		fprintf (stderr, "glaze-find-libgl: Failed to dlsym glClear\n");
		return 1;
	}

	if (dladdr (function, &info) == 0) {
		fprintf (stderr, "glaze-find-libgl: Failed to dladdr glClear\n");
		return 1;
	}

	printf ("%s\n", info.dli_fname);

	return 0;
}
